/**
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "Features/BasicExtractorFactory.h"

#include <opencv2/core.hpp>

namespace MediaVision {
namespace Image {

BasicExtractorFactory::BasicExtractorFactory(
		KeypointType keypointsType,
		DescriptorType descType) :
				__kpType(keypointsType),
				__descType(descType)
{
}

cv::Ptr<FeatureExtractor> BasicExtractorFactory::buildFeatureExtractor()
{
	cv::Ptr<FeatureExtractor> featureExtractor(new (std::nothrow)FeatureExtractor());

	if (featureExtractor != NULL) {
		cv::Ptr<cv::FeatureDetector> detector;
		switch (__kpType) {
		case KT_ORB:
			detector = cv::ORB::create();
			break;
		case KT_GFTT:
			detector = cv::GFTTDetector::create();
			break;
		default:
			LOGE("[%s] Unknown feature detector", __FUNCTION__);
		}

		cv::Ptr<cv::DescriptorExtractor> extractor;
		switch (__descType) {
		case DT_ORB:
			extractor = cv::ORB::create();
			break;
		case DT_BRIEF:
			extractor = cv::xfeatures2d::BriefDescriptorExtractor::create();
			break;
		default:
			LOGE("[%s] Unknown feature extractor", __FUNCTION__);
		}

		featureExtractor->setFeatureDetector(detector, __kpType);
		featureExtractor->setDescriptorExtractor(extractor, __descType);
	}

	return featureExtractor;
}

} /* Image */
} /* MediaVision */
